import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:json_form/element_form.dart';
import 'package:json_form/model_form/adapters/i_element_adapter.dart';
import 'package:json_form/model_form/model_form.dart';

void main(){

  group("Model form tests:", (){

    testWidgets("Test simple model to form conversion", (WidgetTester tester) async {
      GlobalKey<ModelFormState<TestModel>> key = GlobalKey<ModelFormState<TestModel>>();
      TestModel model = TestModel("TestPerson12", "TestPerson12@gmail.com");

      await tester.pumpWidget(MaterialApp(
        home: Scaffold(
          body: Center(
            child: ModelForm<TestModel>(model, [TestAdapter()], testModelToJson, jsonToTestModel, (BuildContext context, dynamic widgets){
              return Column(
                children: <Widget>[
                  widgets["name"],
                  widgets["email"]
                ],
              );
            }, key: key)
          ),
        ),
      ));

      expect(key.currentState.save().name, "TestPerson12");
    });

  });

}

Map<String, dynamic> testModelToJson(TestModel model){
  return {
    "name": model.name,
    "email": model.email
  };
}

TestModel jsonToTestModel(dynamic json){
  return TestModel(
    json["name"],
    json["email"]
  );
}

class TestModel {
  final String name;
  final String email;

  TestModel(this.name, this.email);
}
class TestAdapter extends IElementAdapter<dynamic, FormElement>{

  @override
  bool isMatch(value) {
    if(value.runtimeType == String){
      return true;
    }

    return false;
  }

  @override
  FormElement adapt(from) {
    return TestElement(
        initialValue: from
    );
  }

}
class TestElement extends FormElement{

  final Key key;
  final String _initialValue;
  TextEditingController _controller;

  @override
  get value => _controller.text;
  set value(dynamic value) => _controller.text = value;

  TestElement({this.key, String initialValue}) : _initialValue = initialValue;

  @override
  Widget build(BuildContext context) {
    return TextFormField(
      key: key,
      controller: _controller,
    );
  }

  @override
  dispose() {
    _controller.dispose();
  }

  @override
  initialize() {
    _controller = TextEditingController(text: _initialValue);
  }

}